@@ -180,7 +180,7 @@ NumericVector select_line_connection(
   bool right = false;
   bool bottom = false;
 
-  if ((p1[0] >= b[0]) & (p1[0] <= b[2])) {
+  if ((p1[0] >= b[0]) && (p1[0] <= b[2])) {
     out[0] = p1[0];
   } else if (p1[0] > b[2]) {
     out[0] = b[2];
@@ -190,7 +190,7 @@ NumericVector select_line_connection(
     left = true;
   }
 
-  if ((p1[1] >= b[1]) & (p1[1] <= b[3])) {
+  if ((p1[1] >= b[1]) && (p1[1] <= b[3])) {
     out[1] = p1[1];
   } else if (p1[1] > b[3]) {
     out[1] = b[3];
@@ -622,7 +622,7 @@ Point spring_force(
 std::vector<double> rescale(std::vector<double> v) {
   double min_value = *std::min_element(v.begin(), v.end());
   double max_value = *std::max_element(v.begin(), v.end());
-  for (int i = 0; i < v.size(); i++) {
+  for (long unsigned int i = 0; i < v.size(); i++) {
     v[i] = (v[i] - min_value) / max_value;
   }
   return v;
@@ -675,6 +675,39 @@ DataFrame repel_boxes2(
     force_pull = 1e-6;
   }
 
+  // Try to catch errors.
+  if (n_texts > n_points) {
+    Rcerr << "n_texts is " << n_texts << std::endl;
+    Rcerr << "n_points is " << n_points << std::endl;
+    stop("n_texts > n_points");
+  }
+
+  if (point_size.length() != n_points) {
+    Rcerr << "point_size.length() is " << point_size.length() << std::endl;
+    Rcerr << "n_points is " << n_points << std::endl;
+    stop("point_size.length() != n_points");
+  }
+
+  if (hjust.length() < n_texts) {
+    Rcerr << "hjust.length() is " << hjust.length() << std::endl;
+    Rcerr << "n_texts is " << n_texts << std::endl;
+    stop("hjust.length() < n_texts");
+  }
+  if (vjust.length() < n_texts) {
+    Rcerr << "vjust.length() is " << vjust.length() << std::endl;
+    Rcerr << "n_texts is " << n_texts << std::endl;
+    stop("vjust.length() < n_texts");
+  }
+
+  if (xlim.length() != 2) {
+    Rcerr << "xlim.length() is " << xlim.length() << std::endl;
+    stop("xlim.length() != 2");
+  }
+  if (ylim.length() != 2) {
+    Rcerr << "ylim.length() is " << ylim.length() << std::endl;
+    stop("ylim.length() != 2");
+  }
+
   Point xbounds, ybounds;
   xbounds.x = xlim[0];
   xbounds.y = xlim[1];
@@ -696,7 +729,7 @@ DataFrame repel_boxes2(
   NumericVector r = rnorm(n_texts, 0, force_push);
   std::vector<Box> TextBoxes(n_texts);
   std::vector<Point> original_centroids(n_texts);
-  std::vector<double> TextBoxWidths(n_texts);
+  std::vector<double> TextBoxWidths(n_texts, 0);
   for (int i = 0; i < n_texts; i++) {
     TextBoxes[i].x1 = boxes(i, 0);
     TextBoxes[i].x2 = boxes(i, 2);
@@ -722,7 +755,12 @@ DataFrame repel_boxes2(
   // }
   // Rcout << std::endl;
 
+  // Initialize velocities to zero
   std::vector<Point> velocities(n_texts);
+  for (int i = 0; i < n_texts; i++) {
+    Point v = {0,0};
+    velocities[i] = v;
+  }
   double velocity_decay = 0.7;
 
   Point f, ci, cj;
@@ -804,7 +842,7 @@ DataFrame repel_boxes2(
               direction
             );
           }
-        } else if (too_many_overlaps[j]) {
+        } else if (j < n_texts && too_many_overlaps[j]) {
           // Skip the data points if the size and padding is 0.
           if (point_size[j] == 0 && point_padding_x == 0 && point_padding_y == 0) {
             continue;
@@ -822,13 +860,15 @@ DataFrame repel_boxes2(
             );
           }
         } else {
-          cj = centroid(TextBoxes[j], hjust[j], vjust[j]);
-          // Repel the box from overlapping boxes.
-          if (j < n_texts && overlaps(TextBoxes[i], TextBoxes[j])) {
-            n_overlaps += 1;
-            i_overlaps = true;
-            total_overlaps[i] += 1;
-            f = f + repel_force(ci, cj, force_push, direction);
+          if (j < n_texts) {
+            cj = centroid(TextBoxes[j], hjust[j], vjust[j]);
+            // Repel the box from overlapping boxes.
+            if (overlaps(TextBoxes[i], TextBoxes[j])) {
+              n_overlaps += 1;
+              i_overlaps = true;
+              total_overlaps[i] += 1;
+              f = f + repel_force(ci, cj, force_push, direction);
+            }
           }
 
           // Skip the data points if the size and padding is 0.
@@ -872,13 +912,12 @@ DataFrame repel_boxes2(
 
       // look for line clashes
       if (n_overlaps == 0 || iter % 5 == 0) {
-        for (int j = 0; j < n_points; j++) {
+        for (int j = 0; j < n_texts; j++) {
           cj = centroid(TextBoxes[j], hjust[j], vjust[j]);
           ci = centroid(TextBoxes[i], hjust[i], vjust[i]);
           // Switch label positions if lines overlap
           if (
-            i != j && j < n_texts &&
-            line_intersect(ci, Points[i], cj, Points[j])
+              i != j && line_intersect(ci, Points[i], cj, Points[j])
           ) {
             n_overlaps += 1;
             TextBoxes[i] = TextBoxes[i] + spring_force(cj, ci, 1, direction);
